; ********************************************************************************
; Servo Test Signal Generator
; 
; Release Code 1.0.0 (16/12/2012)
; Pete Griffiths (c) 2012
; info@picprojects.org.uk
; Written and assembled using MPLAB v8.80
; note: source code formated for Tab size = 16 spaces
;
; ********************************************************************************
; Pinout connections 
;
; Supply voltage 4 to 5 volts DC
;                            ___ ___
;                   (+) Vdd -|1 ^ 8|- Vss (-)
;                      n/c  -|2   7|- Variable resistor input (set Servo duty)
;     Servo out active high -|3   6|- Servo out active low
;  MCLR.tie to Vdd via 10K  -|4   5|- n/c
;                            

;
;
; ********************************************************************************


                #include "p12f683.inc"
                #define ADCpresent


              
     __CONFIG       _CP_OFF & _WDT_OFF & _BOD_OFF &_PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON & _CPD_OFF
 
  
                errorlevel -302 ; suppress banksel warning messages
       
; Program data memory variables
; --------------------------------------------
;          
                cblock          0x20 
                                 
 	counter
 	suml
 	sumh

	av0
	av1
	av2
	av3
	av4
	av5
	av6
	av7

	endc

                
;----------------------------------------------------------
; program Constants - DO NOT change them



;----------------------------------------------------------
; GPIO port variables - DO NOT change them
ServoOutH       equ             4               ; out
ServoOutL       equ             1               ; out
Switch	equ	5	; in
                
;----------------------------------------------------------
;
;
; Bank Select macro instructions
#define         setbank0        bcf             STATUS,RP0      ; Sel Bank 0
#define         setbank1        bsf             STATUS,RP0      ; Sel Bank 1
                
                org 0x000
                goto	reset.init
                
                org 0x004
                
   
 
 ; ******************************************************************************************************
; Initialisation and startup code block
; ------------------------------------------------------------------------------------------------------

reset.init
        
                clrf            GPIO

                setbank1                        ; switch to register bank 1
                movlw           ~(1<<ServoOutL | 1<<ServoOutH) ; Set up W for output bits
                movwf           TRISIO
                


                clrf            ANSEL           ; Set ports for digital mode (12F675 / 12F683 only)
	bsf	ANSEL, ANS0	; Set GPIO 0 as analogue input
	bsf	ANSEL, ADCS0	; set conversion clock to 2uS @ 4Mhz clk
                                
                movlw           b'000000001'     ;setup option register
                ;                 ||||||||---- PS0 - Timer 0: Prescalar 1:4
                ;                 |||||||----- PS1 - .......
                ;                 ||||||------ PS2 - .......
                ;                 |||||------- PSA -  Assign prescaler to Timer0
                ;                 ||||-------- TOSE - LtoH edge          (Don't care)
                ;                 |||--------- TOCS - Timer0 uses IntClk (Don't care)
                ;                 ||---------- INTEDG - falling edge RB0 (Don't care)
                ;                 |----------- GPPU - pull-ups enabled 
                movwf           OPTION_REG

                bsf             WPU,Switch       ; enable weak-pull-up for seqSw input
      

      
                setbank0
                
                movlw           0x07            ; load W=7
                movwf           CMCON0          ; disable Comparator on 12F683
  	clrf	ADCON0

	bsf	ADCON0, ADON
  	
      

                ; Clear GPR memory
                ; ---------------------------------------------------------------
                
                movlw           0x20            ; load W with address of first GPR memory
                movwf           FSR             ; save in FSR indirection register
                movwf           0x5F            ; save a non-zero value to last GPR memory (addr 0x5F)
_clrNext        clrf            INDF            ; clear contents of register pointed to by FSR
                incf            FSR,F           ; increment FSR
                movf            0x5F,F          ; Test value in last GPR memory location
                skpz                            ; skip next if last GPR has been cleared
                goto            _clrNext        ; otherwise, repeat until done.

                ; Initialise specific variables
                ; ---------------------------------------------------------------

  	goto	delay

                
new.servo.pulse
              	movlw	(0<<ServoOutL | 1<<ServoOutH)
              	movwf	GPIO
              	
              	movlw	.6	; 1mS
              	movwf	TMR0
	call	average	
                bcf	INTCON, TMR0IF
          	btfss	INTCON, TMR0IF
            	goto	$-1
            	
                bcf	INTCON, TMR0IF
	movfw	suml
              	movwf	TMR0

          	btfss	INTCON, TMR0IF
            	goto	$-1
           
              	movlw	(1<<ServoOutL | 0<<ServoOutH)
              	movwf	GPIO
              		
                
delay	bsf	ADCON0, GO_DONE	;start ADC conversion                       		
              	movlw	.18
              	movwf	counter
              	clrf	TMR0
              	
delay.loop    	bcf	INTCON, TMR0IF
              	btfss	INTCON, TMR0IF
            	goto	$-1
            	
            	decfsz	counter,F
            	goto	delay.loop
            	
            	goto	new.servo.pulse
            	
              		
          
average 	movfw	av6
 	movwf	av7 
 	               
 	movfw	av5
 	movwf	av6   
 	             
	movfw	av4
 	movwf	av5  
 	              
 	movfw	av3
 	movwf	av4
 	
	movfw	av2
 	movwf	av3   
 	             
 	movfw	av1
 	movwf	av2     
 	           
 	movfw	av0
 	movwf	av1     
 	           
 	movfw	ADRESH  
 	movwf	av0
 	
 	movlw	av0
 	movwf	FSR
 	
	clrf	suml
	clrf	sumh
	
sum.next	movfw	INDF
	addwf	suml,F
	skpnc
	incf	sumh,F
	
	incf	FSR,F
	movlw	av7+1
	xorwf	FSR,W
	skpz
	goto	sum.next
        	
        	rrf	sumh,F
        	rrf	suml,F 
        	       
                rrf	sumh,F
        	rrf	suml,F 
        	       
        	rrf	sumh,F
        	rrf	suml,F 
        	return

                
                


	org 0x2100
;               ------------------------------------------------------------------------
                ; Embed code revision into EEPROM memory
                 de "ServoTester683.asm 16/12/2012 R1.0.0"  
                 de "(c) Pete Griffiths 2006-2012 "
                 de "info@picprojects.org.uk"
;               ------------------------------------------------------------------------
                
                ;fill 00, 0x3FF-$
                
                end

